/***********************************************************
    Module Name: Main.c
    Module Date: 04/10/2004
    Module Auth: John Orlando 
    Copyright (c) 2004 John Orlando  All Rights Reserved 

    Description:  This module is responsible for providing
    the entry point to the code through the "main" function.
    ***********************************************************/

/*	Includes */
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "UIMgr.h"
#include "UartInterface.h"
#include "I2CInterface.h"
#include "CamInterface.h"
#include "DebugInterface.h"
#include "FrameMgr.h"
#include "CommonDefs.h"
#include "CamConfig.h"
#include "Executive.h"
#include "Utility.h"

/* 	Local Structures and Typedefs */

/*  Extern Variables */

/*  Definitions */

/***********************************************************
    Function Name: main
    Function Description: This function provides the entry
    point into AVRcam application.
    Inputs:  none
    Outputs: int
    ***********************************************************/
int main(void)
{
    /* initialize all of the interface modules */
    DebugInt_init();
    UartInt_init();
    I2CInt_init();
    CamInt_init();

    /* initialize the remaining modules that will process
        data...interrupts need to be on for these */
    ENABLE_INTS();
    CamConfig_init();
    UIMgr_init();
    FrameMgr_init();

    /* provide a short delay for the camera to stabilize before
        we let the executive start up */
    Utility_delay(1000);

    /* the rest of the application will be under the
        control of the Executive.  */
    Exec_run();

    /* this should never be reached */
    return(0);
}

