package img.io;

import java.io.*;

public class AVRInputStream extends FilterInputStream {

   private static final int NEW_LINE = '\r';
   private static final int END_DUMP_DATA = 0x0F;
   private static final int END_TRACK_DATA = 0xFF;

   private static final int SEARCHING_STATE = 0x00;
   private static final int DUMP_FRAME_STATE = 0x01;
   private static final int ACCEPTING_STATE = 0x02;

   private BufferedOutputStream out;

   public AVRInputStream(InputStream in) {
      super(in);
      try {
         out = new BufferedOutputStream(new FileOutputStream("Testing.out"));
      } catch(IOException ioe) {
         ioe.printStackTrace();
      }
   }

   public int read(byte[] data, int off, int len) throws IOException {

      int terminator = NEW_LINE;
      int state = SEARCHING_STATE;

      int value = -1;
      int bytesRead = 0;
      boolean cont = false;

      do {

         value = read();

         if(value != -1) {

            switch(state) {
               case SEARCHING_STATE:
                  //System.out.println("(" + Integer.toHexString(value & 0xFF) + ") ");
                  out.write(value & 0xFF);
                  data[off] = (byte)value;
                  if(value == 0x0A) {
                     terminator = END_TRACK_DATA;
                     state = ACCEPTING_STATE;
                     bytesRead++;
                  } else if(value == 0x0B) {
                     terminator = END_DUMP_DATA;
                     state = DUMP_FRAME_STATE;
                     bytesRead++;
                  } else if(value == 'A' || value == 'N') {
                     terminator = NEW_LINE;
                     state = ACCEPTING_STATE;
                     bytesRead++;
                  }
                  cont = false;
                  break;
               case DUMP_FRAME_STATE:
                  data[off + bytesRead++] = (byte)value;
                  out.write(value & 0xFF);
                  state = ACCEPTING_STATE;
                  cont = true;
                  break;
               case ACCEPTING_STATE:
                  //System.out.println(Integer.toHexString(value & 0xFF) + " ");
                  out.write(value & 0xFF);
                  data[off + bytesRead++] = (byte)value;
                  cont = false;
                  break;
            }

         }


      } while(cont || value != terminator);

      return bytesRead;
   }

}
