package img.swing;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.prefs.*;
import javax.swing.*;

import javax.comm.*;

import log.swing.*;
import log.util.*;

import img.connection.*;
import img.connection.event.*;
import img.device.event.*;

import img.lang.ImageSystem;

public class JImageFrame extends JFrame {

   public static void main(String[] args) {
      /*
      try {
         UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      } catch(Exception e) {
      }
      */

      try {
         if(args.length == 1) {
            if(args[0].equals("-r")) {
               ImageSystem.PREFS.clear();
            }
         }
      } catch(BackingStoreException bse) {
         System.out.println("Could not clear preferences: " + bse.getMessage());
      }
      SwingUtilities.invokeLater(new Runnable() {
         public void run() {
            JFrame frame = new JImageFrame();
            frame.show();
         }
      });
   }

   private JFrame logF;

   private JSerialPanel serialP;
   private JRegisterPanel registersP;
   private JMessagePanel messageP;
   private JColorMapPanel colorMapP;
   private JFrame colorMapF;


   private JDesktopPane desktop;
   private JInternalFrame captureF;
   private JTrackingInternalFrame trackingF;

   private AbstractButton viewLogB;
   private AbstractButton viewMessagesB;
   private AbstractButton viewColorMapB;
   private AbstractButton trackingB;

   private Action connectAction;
   private Action disconnectAction;
   private Action serialParamsAction;
   private Action setRegistersAction;
   private Action viewMsgsAction;
   private Action pingAction;
   private Action resetAction;
   private Action captureAction;
   private Action trackingAction;

   public JImageFrame() {
      super("AVRcamVIEW");

      serialP = new JSerialPanel();
      registersP = new JRegisterPanel();

      messageP = new JMessagePanel();
      colorMapP = new JColorMapPanel(messageP);

      Dimension screen = getToolkit().getScreenSize();

      JLogApplet logA = new JLogApplet();
      logA.init(false);
      ImageSystem.LOG.addHandler(new LogHandler(logA.getTableModel(), ImageSystem.LOG.getLevel()));

      logF = logA.createFrame();
      logF.setBounds(0, 0, screen.width * 3 / 4, screen.height / 2);

      try {
         createActions();
      } catch(NoSuchMethodException nsme) {
         throw new NoSuchMethodError(nsme.getMessage());
      }

      desktop = new JDesktopPane();
      desktop.setBackground(new Color(0x005C5C));

      setJMenuBar(createMenuBar());
      getContentPane().add(createToolBar(), BorderLayout.NORTH);
      getContentPane().add(desktop, BorderLayout.CENTER);
      getContentPane().add(messageP, BorderLayout.EAST);

      setBounds(screen.width / 8, screen.height / 8,
                screen.width * 3 / 4, screen.height * 3 / 4);

      colorMapF = new JFrame("Color Map");
      colorMapF.addWindowListener(new ColorMapWindowHandler(colorMapF, viewColorMapB));
      colorMapF.getContentPane().add(colorMapP, BorderLayout.CENTER);
      colorMapF.pack();

      addWindowListener(new WindowHandler(this));
      logF.addWindowListener(new LogWindowHandler(viewLogB));

      try {
         Action action = new ProxyAction(ImageSystem.DEVICE, "simulateNCK", "Simulate NCK");

         // attach the action to the Ctrl+N key
         getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT)
                      .put(KeyStroke.getKeyStroke(KeyEvent.VK_N,
                                                  InputEvent.CTRL_MASK),
                           action.getValue(Action.NAME));
         getRootPane().getActionMap().put(action.getValue(Action.NAME), action);

         action = new ProxyAction(ImageSystem.DEVICE, "simulateACK", "Simulate ACK");

         // attach the action to the Ctrl+A key
         getRootPane().getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT)
                      .put(KeyStroke.getKeyStroke(KeyEvent.VK_A,
                                                  InputEvent.CTRL_MASK),
                           action.getValue(Action.NAME));
         getRootPane().getActionMap().put(action.getValue(Action.NAME), action);

      } catch(NoSuchMethodException nsme) {
         throw new NoSuchMethodError(nsme.getMessage());
      }

      SwingUtilities.invokeLater(new Startup());

   }

   private void createActions() throws NoSuchMethodException {
      serialParamsAction = new ProxyAction(this, "showSerialDialog", "Configure Serial", 'S');
      setRegistersAction = new ProxyAction(this, "showRegisterDialog", "Set Registers", 'R');
      setRegistersAction.setEnabled(false);

      connectAction = new ProxyAction(this, "connect", "Connect", 'C');
      disconnectAction = new ProxyAction(this, "disconnect", "Disconnect", 'D');
      disconnectAction.setEnabled(false);

      pingAction = new ProxyAction(this, "ping", "Ping", 'P');
      pingAction.setEnabled(false);

      resetAction = new ProxyAction(this, "reset", "Reset", 'R');
      resetAction.setEnabled(false);

      captureAction = new ProxyAction(this, "capture", "Capture", 'C');
      captureAction.setEnabled(false);

      trackingAction = new ProxyAction(this, "tracking", "Enable Tracking", 'E');
      trackingAction.setEnabled(false);
      trackingB = new JToggleButton(trackingAction);
   }

   private JMenuBar createMenuBar() {
      JMenuBar menubar = new JMenuBar();

      JMenu fileM = new JMenu("File");
      JMenu viewM = new JMenu("View");
      JMenu deviceM = new JMenu("Device");
      JMenu helpM = new JMenu("Help");

      fileM.setMnemonic('f');
      viewM.setMnemonic('v');
      deviceM.setMnemonic('d');
      helpM.setMnemonic('h');

      try {
         fileM.add(new ProxyAction(this, "openBayer", "Open Bayer Image", 'B'));
         fileM.add(new ProxyAction(this, "openTracking", "Open Tracking Data", 'T'));
         fileM.addSeparator();
         fileM.add(serialParamsAction);
         fileM.add(setRegistersAction);
         fileM.addSeparator();
         fileM.add(new ProxyAction(this, "close", "Exit", 'X'));

         viewColorMapB = new JCheckBoxMenuItem(new ProxyAction(this, "viewColorMap", true, "Show Color Map", 'C'));
         viewM.add(viewColorMapB);

         viewMessagesB = new JCheckBoxMenuItem(new ProxyAction(this, "viewMessages", true, "Show Messages", 'M'));
         viewM.add(viewMessagesB);

         viewLogB = new JCheckBoxMenuItem(new ProxyAction(this, "viewLog", true, "Show Log", 'L'));
         viewM.add(viewLogB);

         deviceM.add(connectAction);
         deviceM.add(disconnectAction);

         helpM.add(new ProxyAction(this, "about", "About", 'A'));
      } catch(NoSuchMethodException nsme) {
         throw new NoSuchMethodError(nsme.getMessage());
      }

      menubar.add(fileM);
      menubar.add(viewM);
      menubar.add(deviceM);
      menubar.add(helpM);

      return menubar;
   }

   private JToolBar createToolBar() {
      JToolBar toolbar = new JToolBar();

      toolbar.add(connectAction);
      toolbar.add(disconnectAction);
      toolbar.addSeparator();
      toolbar.add(serialParamsAction);
      toolbar.add(setRegistersAction);
      toolbar.addSeparator();
      toolbar.add(pingAction);
      toolbar.add(resetAction);
      toolbar.add(captureAction);
      toolbar.add(trackingB);

      toolbar.setFloatable(false);

      return toolbar;
   }

   protected JRootPane createRootPane() {
      // create a new JRootPane
      JRootPane rootPane = new JRootPane() {

         /**
          * Create a new Glass Pane Glass Pane that when visible it captures
          * all the mouse events, plays a "beep" when the mouse is clicked
          * when the Glass Pane is visible, and changes the mouse depending
          * on if the Glass Pane is visible or not.
          * @return The Glass Pane
          */
         protected Component createGlassPane() {
            JComponent c = new JPanel() {

               /**
                * Overridden to change the cursor to a busy cursor when visible
                * and a default cursor when not visible.
                * @param visible True to show the Glass Pane, false to not
                * show the Glass Pane.
                */
               public void setVisible(boolean visible) {
                  super.setVisible(visible);
                  if(visible) {
                     // this is possible if it this panel has not been added
                     // to a JRootPane yet
                     if(getRootPane() != null) {
                        // change the cursor to a wait/busy cursor (hour glass)
                        getRootPane().setCursor(
                           Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                        ImageSystem.LOG.finest("Glass Pane SHOWING... ");
                     }
                  } else {
                     // this is possible if it this panel has not been added
                     // to a JRootPane yet
                     if(getRootPane() != null) {
                        // chage the cursor back to the default cursor
                        getRootPane().setCursor(Cursor.getDefaultCursor());
                        ImageSystem.LOG.finest("Glass Pane HIDDEN... ");
                     }
                  }
               }
            };
            c.setName(this.getName() + ".glassPane");
            c.setVisible(false);
            ((JPanel)c).setOpaque(false);
            // add a mouse listener to capture all the mouse events and play
            // a "beep" when the mouse is pressed.
            c.addMouseListener(new MouseAdapter() {
               public void mousePressed(MouseEvent me) {
                  Toolkit.getDefaultToolkit().beep();
               }
            });
            return c;
         }
      };
      rootPane.setOpaque(true);
      return rootPane;
   }

   public void openBayer() {
      int option = ImageSystem.FILE_CHOOSER.showOpenDialog(getRootPane());
      if(option == JFileChooser.APPROVE_OPTION) {
         JInternalFrame iFrame = new JCaptureInternalFrame(messageP, colorMapP, ImageSystem.FILE_CHOOSER.getSelectedFile());
         iFrame.pack();
         iFrame.setLocation(10, 10);
         desktop.add(iFrame);
         iFrame.setVisible(true);
      }
   }

   public void openTracking() {
      int option = ImageSystem.FILE_CHOOSER.showOpenDialog(getRootPane());
      if(option == JFileChooser.APPROVE_OPTION) {
         try {
            JInternalFrame iFrame = new JTrackingInternalFrame(ImageSystem.FILE_CHOOSER.getSelectedFile());
            iFrame.pack();
            iFrame.setLocation(10, 10);
            desktop.add(iFrame);
            iFrame.setVisible(true);
         } catch(FileNotFoundException fnfe) {
            JOptionPane.showMessageDialog(getRootPane(), "File Not Found", fnfe.getMessage(), JOptionPane.ERROR_MESSAGE);
         } catch(IOException ioe) {
            JOptionPane.showMessageDialog(getRootPane(), "I/O Exception", ioe.getMessage(), JOptionPane.ERROR_MESSAGE);
         }
      }
   }

   public void showSerialDialog() {
      if(ImageSystem.DEVICE.getConnection() != null) {
         showSerialDialog(((SerialConnection)ImageSystem.DEVICE.getConnection()).getSerialParams());
      } else {
         showSerialDialog(null);
      }
   }

   public void showSerialDialog(SerialParams params) {
      int option = serialP.showDialog(this, params);
      if(option == JSerialPanel.OK_OPTION) {
         try {
            if(ImageSystem.DEVICE.isConnected()) {
               ((SerialConnection)ImageSystem.DEVICE.getConnection()).setSerialParams(serialP.getSerialParameters());
            }
            saveConnectionPrefs(null, serialP.getSerialParameters());
         } catch(UnsupportedCommOperationException ucoe) {
            ImageSystem.LOG.severe(ucoe.getMessage());
         }
      }
   }

   public void showRegisterDialog() {
      int option = registersP.showDialog(this);
      if(option == JRegisterPanel.OK_OPTION && registersP.getRegisters().size() > 0) {

         DataListener handler = null;
         try {
            handler = new SetRegistersHandler();
            ImageSystem.DEVICE.addDataListener(handler);
            getRootPane().getGlassPane().setVisible(true);
            ImageSystem.DEVICE.sendSetRegisters(registersP.getRegisters());
            messageP.append("Set Registers");
         } catch(IOException ioe) {
            ImageSystem.DEVICE.removeDataListener(handler);
            getRootPane().getGlassPane().setVisible(false);
            ImageSystem.LOG.severe(ioe.getMessage());
            messageP.append("Set Registers not sent");
         }
      }
   }

   public void close() {

      ImageSystem.PREFS.putBoolean("img.device.connected", ImageSystem.DEVICE.isConnected());

      if(ImageSystem.DEVICE.isConnected()) {
         saveConnectionPrefs((SerialConnection)ImageSystem.DEVICE.getConnection());
         disconnect();
      }

      ImageSystem.PREFS.putBoolean("img.log.showing", logF.isVisible());
      ImageSystem.PREFS.putBoolean("img.message.showing", messageP.isVisible());
      ImageSystem.PREFS.putBoolean("img.colormap.showing", colorMapF.isVisible());

      System.exit(0);
   }

   private void saveConnectionPrefs(SerialConnection con) {
      saveConnectionPrefs(con.getComPort(), con.getSerialParams());
   }

   private void saveConnectionPrefs(String comPort, SerialParams params) {
      if(comPort != null) {
         ImageSystem.PREFS.put("img.com.port", comPort);
      }
      try {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         ObjectOutputStream oOut = new ObjectOutputStream(out);

         oOut.writeObject(params);
         oOut.close();

         byte[] serialParams = out.toByteArray();

         ImageSystem.PREFS.putByteArray("img.com.params", serialParams);
      } catch(IOException ioe) {
         ioe.printStackTrace();
         ImageSystem.LOG.warning("Could not save serial parameters: " + ioe.getMessage());
      }
   }

   private static class ColorMapWindowHandler extends WindowAdapter {

      private JFrame frame;
      private AbstractButton viewB;

      public ColorMapWindowHandler(JFrame frame, AbstractButton viewB) {
         this.frame = frame;
         this.viewB = viewB;
      }

      public void windowClosed(WindowEvent we) {
         viewB.setSelected(false);
      }

      public void windowClosing(WindowEvent we) {
         frame.dispose();
      }
   }

   public void viewColorMap(ActionEvent ae) {
      colorMapF.setVisible(((AbstractButton)ae.getSource()).isSelected());
   }

   public void viewLog(ActionEvent ae) {
      logF.setVisible(((AbstractButton)ae.getSource()).isSelected());
   }

   public void viewMessages(ActionEvent ae) {
      messageP.setVisible(((AbstractButton)ae.getSource()).isSelected());
   }

   public void connect() {
      String[] ports = SerialConnection.getSerialPorts();
      if(ports.length > 0) {
         Object option = JOptionPane.showInputDialog(this, "Select a COM Port:", "Serial Port",
                                                  JOptionPane.QUESTION_MESSAGE,
                                                  null,
                                                  ports,
                                                  ImageSystem.PREFS.get("img.com.port", ports[0]));
         if(option != null) {
            connect(option.toString(), serialP.getSerialParameters());
         }
      } else {
         JOptionPane.showMessageDialog(this, "No Serial Ports Available", "No Serial Ports",
                                       JOptionPane.ERROR_MESSAGE);
      }
   }

   public void connect(String comPort, SerialParams params) {
      SerialConnection con = new SerialConnection(comPort, params);
      con.addConnectionListener(new ConnectionHandler());
      ImageSystem.DEVICE.setConnection(con);
      try {
         ImageSystem.DEVICE.connect();
      } catch(Exception ioe) {
         JOptionPane.showMessageDialog(getRootPane(),
                                       ioe.getMessage(),
                                       "Connect Error",
                                       JOptionPane.ERROR_MESSAGE);
         ImageSystem.LOG.severe(ioe.getMessage());
      }
   }

   public void disconnect() {
      ImageSystem.DEVICE.disconnect();
   }

   public void ping() {
      DataListener handler = null;
      try {
         handler = new PingHandler();
         ImageSystem.DEVICE.addDataListener(handler);
         getRootPane().getGlassPane().setVisible(true);
         ImageSystem.DEVICE.sendPing();
         messageP.append("Ping");
      } catch(IOException ioe) {
         ImageSystem.DEVICE.removeDataListener(handler);
         getRootPane().getGlassPane().setVisible(false);
         ImageSystem.LOG.severe(ioe.getMessage());
         messageP.append("Ping not sent");
      }
   }

   public void reset() {
      DataListener handler = null;
      try {
         handler = new ResetHandler();
         ImageSystem.DEVICE.addDataListener(handler);
         getRootPane().getGlassPane().setVisible(true);
         ImageSystem.DEVICE.sendReset();
         messageP.append("Reset");
      } catch(IOException ioe) {
         ImageSystem.DEVICE.removeDataListener(handler);
         getRootPane().getGlassPane().setVisible(false);
         ImageSystem.LOG.severe(ioe.getMessage());
         messageP.append("Reset not sent");
      }
   }

   public void capture() {
      JCaptureInternalFrame captureF = new JCaptureInternalFrame(messageP, colorMapP);
      Insets insets = captureF.getInsets();
      captureF.pack();
      int frameCount = desktop.getAllFrames().length;
      captureF.setLocation(10 * frameCount, 10 * frameCount);
      desktop.add(captureF);

      captureF.setVisible(true);
   }

   public void tracking() {
      DataListener handler = new TrackingHandler();
      if(trackingB.isSelected()) {
         try {
            ImageSystem.DEVICE.addDataListener(handler);
            ImageSystem.DEVICE.sendEnableTracking();
            messageP.append("Enable Tracking");
         } catch(IOException ioe) {
            ImageSystem.DEVICE.removeDataListener(handler);
            ImageSystem.LOG.severe(ioe.getMessage());
            messageP.append("Enable Tracking not sent");
         }
      } else {
         try {
            ImageSystem.DEVICE.addDataListener(handler);
            ImageSystem.DEVICE.sendDisableTracking();
            messageP.append("Disable Tracking");
         } catch(IOException ioe) {
            ImageSystem.DEVICE.removeDataListener(handler);
            ImageSystem.LOG.severe(ioe.getMessage());
            messageP.append("Disable Tracking not sent");
         }
      }
   }

   public void about() {
      new JAboutDialog(this, messageP).show();
   }

   private class SetRegistersHandler extends DataAdapter {

      public void ack() {
         ImageSystem.DEVICE.removeDataListener(this);
         getRootPane().getGlassPane().setVisible(false);
      }

      public void nck() {
         ImageSystem.DEVICE.removeDataListener(this);
         getRootPane().getGlassPane().setVisible(false);
         JOptionPane.showMessageDialog(JImageFrame.this, "Set Register NCK Received", "NCK Received", JOptionPane.ERROR_MESSAGE);
      }

   }

   private class PingHandler extends DataAdapter {

      public void ack() {
         ImageSystem.DEVICE.removeDataListener(this);
         getRootPane().getGlassPane().setVisible(false);
      }

      public void nck() {
         ImageSystem.DEVICE.removeDataListener(this);
         getRootPane().getGlassPane().setVisible(false);
         JOptionPane.showMessageDialog(JImageFrame.this, "Ping NCK Received", "NCK Received", JOptionPane.ERROR_MESSAGE);
      }

   }

   private class ResetHandler extends DataAdapter {

      public void ack() {
         ImageSystem.DEVICE.removeDataListener(this);
         getRootPane().getGlassPane().setVisible(false);
      }

      public void nck() {
         ImageSystem.DEVICE.removeDataListener(this);
         getRootPane().getGlassPane().setVisible(false);
         JOptionPane.showMessageDialog(JImageFrame.this, "Reset NCK Received", "NCK Received", JOptionPane.ERROR_MESSAGE);
      }

   }

   private class TrackingHandler extends DataAdapter {
      public void ack() {
         ImageSystem.DEVICE.removeDataListener(this);
         if(trackingB.isSelected()) {
            trackingB.setText("Disable Tracking");
            serialParamsAction.setEnabled(false);
            setRegistersAction.setEnabled(false);
            pingAction.setEnabled(false);
            resetAction.setEnabled(false);
            captureAction.setEnabled(false);

            trackingF = new JTrackingInternalFrame();
            Insets insets = trackingF.getInsets();
            trackingF.pack();
            trackingF.setLocation(10, 10);

            desktop.add(trackingF);

            trackingF.setVisible(true);
            trackingF.startTracking();

         } else {
            trackingB.setText("Enable Tracking");
            serialParamsAction.setEnabled(true);
            setRegistersAction.setEnabled(true);
            pingAction.setEnabled(true);
            resetAction.setEnabled(true);
            captureAction.setEnabled(true);

            trackingF.stopTracking();
            desktop.getDesktopManager().closeFrame(trackingF);
         }
      }
      public void nck() {
         ImageSystem.DEVICE.removeDataListener(this);
         if(trackingB.isSelected()) {
            trackingB.setSelected(false);
            JOptionPane.showMessageDialog(JImageFrame.this, "Enable Tracking NCK Received", "NCK Received", JOptionPane.ERROR_MESSAGE);
         } else {
            trackingB.setSelected(true);
            JOptionPane.showMessageDialog(JImageFrame.this, "Disable Tracking NCK Received", "NCK Received", JOptionPane.ERROR_MESSAGE);
         }
      }
   }

   private class Startup implements Runnable {

      public void run() {

         boolean logVisible = ImageSystem.PREFS.getBoolean("img.log.showing", false);
         logF.setVisible(logVisible);
         viewLogB.setSelected(logVisible);

         boolean messagesVisible = ImageSystem.PREFS.getBoolean("img.message.showing", true);
         messageP.setVisible(messagesVisible);
         viewMessagesB.setSelected(messagesVisible);

         boolean colorMapVisible = ImageSystem.PREFS.getBoolean("img.colormap.showing", false);
         viewColorMapB.setSelected(colorMapVisible);
         if(colorMapVisible) {
            viewColorMap(new ActionEvent(viewColorMapB, ActionEvent.ACTION_PERFORMED, viewColorMapB.getText()));
         }

         try {

            String comPort = ImageSystem.PREFS.get("img.com.port", "COM1");
            byte[] serialParams = ImageSystem.PREFS.getByteArray("img.com.params", null);

            ImageSystem.LOG.config("Prefs Com Port: " + comPort);
            SerialParams params = null;
            if(serialParams != null) {
               ObjectInputStream in = new ObjectInputStream(
                                         new ByteArrayInputStream(serialParams));
               params = (SerialParams)in.readObject();
               in.close();

               ImageSystem.LOG.config("Prefs Com Params: " + params);

               if(ImageSystem.PREFS.getBoolean("img.device.connected", false)) {
                  connect(comPort, params);
               }
            }
         } catch(Exception e) {
            e.printStackTrace();
            ImageSystem.LOG.warning("Could not read serial params: " + e.getMessage());
         }

      }

   }

   private static class WindowHandler extends WindowAdapter {

      private JImageFrame frame;

      public WindowHandler(JImageFrame frame) {
         this.frame = frame;
      }

      public void windowClosing(WindowEvent we) {
         frame.dispose();
      }

      public void windowClosed(WindowEvent we) {
         frame.close();
      }

   }

   private static class LogWindowHandler extends WindowAdapter {

      private AbstractButton button;

      public LogWindowHandler(AbstractButton button) {
         this.button = button;
      }

      public void windowClosing(WindowEvent we) {
         button.setSelected(false);
      }

   }

   private class ConnectionHandler implements ConnectionListener {
      public void connected(ConnectionEvent ce) {

         SerialConnection con = (SerialConnection)ce.getSource();

         saveConnectionPrefs(con.getComPort(), con.getSerialParams());

         connectAction.setEnabled(false);
         disconnectAction.setEnabled(true);
         setRegistersAction.setEnabled(true);
         pingAction.setEnabled(true);
         resetAction.setEnabled(true);
         captureAction.setEnabled(true);
         trackingAction.setEnabled(true);
      }
      public void disconnected(ConnectionEvent ce) {
         connectAction.setEnabled(true);
         disconnectAction.setEnabled(false);
         setRegistersAction.setEnabled(false);
         pingAction.setEnabled(false);
         resetAction.setEnabled(false);
         captureAction.setEnabled(false);
         trackingAction.setEnabled(false);
      }
   }

}
