package img.swing;

import java.awt.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.text.*;
import java.util.*;
import javax.swing.*;

import img.device.event.*;

import img.lang.*;

public class JTrackingPanel extends JPanel {

   private static final DateFormat DATE_FORMAT;

   static {
      DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd hh.mm.ss");
   }

   private ColorBlob[] blobs;
   private DataListener dataHandler;

   private Dimension preferredSize;

   private boolean recording;

   public JTrackingPanel() {
      super(null);

      recording = false;

      setBackground(Color.WHITE);
      dataHandler = new TrackingHandler();

      preferredSize = null;
   }

   private FileOutputStream outStream;
   private FileChannel outChannel;

   public void startRecording() throws FileNotFoundException {
      File recordFile = new File(DATE_FORMAT.format(new Date()) + ".trk");
      outStream = new FileOutputStream(recordFile);
      outChannel = outStream.getChannel();

      recording = true;

      ImageSystem.LOG.info("Started Recording");
   }

   public void stopRecording() throws IOException {

      recording = false;

      outStream.close();
      outChannel.close();

      ImageSystem.LOG.info("Stopped Recording");

   }

   public void play() {
   }

   public void pause() {
   }

   public void stop() {
   }

   public void startTracking() {
      ImageSystem.DEVICE.addDataListener(dataHandler);
   }

   public void stopTracking() {
      ImageSystem.DEVICE.removeDataListener(dataHandler);
   }

   public Dimension getMinimumSize() {
      return getPreferredSize();
   }

   public Dimension getPreferredSize() {
      if(preferredSize == null) {
         Insets insets = insets();
         preferredSize = new Dimension(insets.left + ImageSystem.IMAGE_WIDTH + insets.right,
                                       insets.top + ImageSystem.IMAGE_HEIGHT + insets.bottom);
      }

      return preferredSize;
   }

   public Dimension getMaximumSize() {
      return getPreferredSize();
   }

   public void paintComponent(Graphics g) {

      super.paintComponent(g);

      if(blobs != null) {
         Dimension size = getSize();
         Insets insets = insets();

         int width = size.width - insets.left - insets.right;
         int height = size.height - insets.top - insets.bottom;

         Image bufferedImage = createImage(width, height);

         Graphics2D bufferGraphics = (Graphics2D)bufferedImage.getGraphics();
         bufferGraphics.scale(width / (double)ImageSystem.IMAGE_WIDTH, height / (double)ImageSystem.IMAGE_HEIGHT);

         for(int i = 0; i < blobs.length; i++) {
            ColorBlob blob = blobs[i];
            if(blob != null) {
               bufferGraphics.setColor(ImageSystem.DEVICE.getMapColors()[blob.colorIndex]);
               bufferGraphics.fillRect(blob.center.x - 2, blob.center.y - 2, 4, 4);
               bufferGraphics.setColor(Color.BLACK);
               bufferGraphics.drawRect(blob.bounds.x, blob.bounds.y,
                                       blob.bounds.width, blob.bounds.height);
/*
               bufferGraphics.setColor(ImageSystem.DEVICE.getMapColors()[blob.colorIndex]);
               bufferGraphics.fillOval(blob.bounds.x, blob.bounds.y,
                                       blob.bounds.width, blob.bounds.height);
               bufferGraphics.setColor(Color.BLACK);
               bufferGraphics.drawOval(blob.bounds.x, blob.bounds.y,
                                       blob.bounds.width, blob.bounds.height);
*/
            }
         }

         g.drawImage(bufferedImage, insets.left, insets.top, this);

      }

   }

   public void setTrackingData(ByteBuffer data) {
      blobs = new ColorBlob[data.get() & 0xFF];

      for(int i = 0; i < blobs.length; i++) {
         blobs[i] = new ColorBlob(data);
      }

      repaint();
   }

   private class TrackingHandler extends DataAdapter {

      public void trackingData(ByteBuffer data) {
         setTrackingData(data);

         if(recording) {
            data.reset();
            try {
               outChannel.write(data);
            } catch(IOException ioe) {
               ImageSystem.LOG.warning("TRACKING: " + ioe.getMessage());
            }
         }

      }

   }

   private static class ColorBlob {

      public final int colorIndex;
      public final Point center;
      public final Rectangle bounds;

      public ColorBlob(ByteBuffer data) {
         colorIndex = data.get() & 0xFF;
         center = new Point(data.get() & 0xFF, data.get() & 0xFF);
         int x = data.get() & 0xFF;
         int y = data.get() & 0xFF;
         int width = (data.get() & 0xFF) - x;
         int height = (data.get() & 0xFF) - y;
         bounds = new Rectangle(x, y, width, height);
      }

      public String toString() {
         return "ColorBlob: " + colorIndex + " (" + center.x + ", " + center.y + ") " +
                " [" + bounds.x + ", " + bounds.y + ", " + bounds.width + ", " + bounds.height + "]";
      }


   }

}
