package log.swing.table;

import java.awt.*;
import java.text.*;
import java.util.*;
import java.util.logging.*;
import javax.swing.*;
import javax.swing.table.DefaultTableCellRenderer;

public class LogTableCellRenderer extends DefaultTableCellRenderer {

   private Color defaultBackground;
   private Color defaultForeground;

   private static final DateFormat DATE_FORMAT;

   static {

      DATE_FORMAT = new SimpleDateFormat("MM/dd/yyyy hh:mm:ss.SSS");

   }

   public LogTableCellRenderer() {
      super();
      defaultBackground = getBackground();
      defaultForeground = getForeground();
   }

   public Component getTableCellRendererComponent(JTable table, Object value,
      boolean isSelected, boolean hasFocus, int row, int column) {

      String text = null;

      switch(column) {
         case 0:
            text = DATE_FORMAT.format(new Date(((LogRecord)value).getMillis()));
            break;
         case 1:
            text = ((LogRecord)value).getLevel().toString();
            break;
         case 2:
            LogRecord record = (LogRecord)value;

            if(record.getMessage() == null) {
               text =  "No Message...";
            } else {
               text = record.getMessage();
            }

            if(!(text.startsWith("Sending") || text.startsWith("Received"))) {
               StringBuffer buffer = new StringBuffer(text);

               buffer.append(" ");
               if(record.getSourceClassName() != null) {
                  buffer.append(record.getSourceClassName());
               }

               if(record.getSourceMethodName() != null) {
                  buffer.append(":");
                  buffer.append(record.getSourceMethodName());
               }
               buffer.append(" ");

               text = buffer.toString();
            }

            break;
      }

      super.getTableCellRendererComponent(table, text, isSelected, hasFocus, row, column);

      if(!isSelected) {
         if(((LogRecord)value).getLevel().equals(Level.SEVERE)) {
            setBackground(Color.RED.darker());
            setForeground(Color.WHITE);
         } else if(((LogRecord)value).getLevel().equals(Level.WARNING)) {
            setBackground(Color.ORANGE);
            setForeground(Color.BLACK);
         } else if(((LogRecord)value).getLevel().equals(Level.INFO)) {
            setBackground(Color.YELLOW);
            setForeground(Color.BLACK);
         } else if(((LogRecord)value).getLevel().equals(Level.CONFIG)) {
            setBackground(Color.CYAN);
            setForeground(Color.BLACK);
         } else {
            setBackground(defaultBackground);
            setForeground(defaultForeground);
         }
      }

      if(column != 2) {
         setHorizontalAlignment(JLabel.CENTER);
      } else {
         setHorizontalAlignment(JLabel.LEFT);
      }

      return this;

   }
}
