package log.swing.table;

import java.util.*;
import java.util.logging.*;
import javax.swing.table.*;

public class LogTableModel extends AbstractTableModel {

   public static final String[] COLUMN_NAMES = {
      "Date",
      "Level",
      "Message"
   };

   private List allRecords;
   private List filteredRecords;

   private Level level;
   private boolean selectedLabel;

   public LogTableModel(Level filterLevel) {
      this(filterLevel, false);
   }

   public LogTableModel(Level filterLevel, boolean selectedLabel) {
      super();
      allRecords = new ArrayList(10);
      filteredRecords = new ArrayList(10);
      this.level = filterLevel;
   }

   public void setFilter(Level level) {
      this.level = level;
      filteredRecords = new ArrayList(allRecords.size());
      Iterator i = allRecords.iterator();
      LogRecord record;
      while(i.hasNext()) {
         record = (LogRecord)i.next();
         if(selectedLabel) {
            if(level.intValue() == record.getLevel().intValue()) {
               filteredRecords.add(record);
            }
         } else {
            if(level.intValue() <= record.getLevel().intValue()) {
               filteredRecords.add(record);
            }
         }
      }
      fireTableDataChanged();
   }

   public void setOnlyShowSelectedLevel(boolean selectedLabel) {
      this.selectedLabel = selectedLabel;
      setFilter(level);
   }

   public void addRecord(LogRecord record) {
      allRecords.add(record);
      if(selectedLabel) {
         if(level.intValue() == record.getLevel().intValue()) {
            filteredRecords.add(record);
         }
      } else {
         if(level.intValue() <= record.getLevel().intValue()) {
            filteredRecords.add(record);
         }
      }
      this.fireTableRowsInserted(filteredRecords.size() - 1, filteredRecords.size() - 1);
   }

   public void clear() {
      allRecords.clear();
      filteredRecords.clear();
      fireTableDataChanged();
   }

   public int getRowCount() {
      return filteredRecords.size();
   }

   public int getColumnCount() {
      return COLUMN_NAMES.length;
   }

   public String getColumnName(int col) {
      return COLUMN_NAMES[col];
   }

   public Class getColumnClass(int col) {
      return String.class;
   }

   public Object getValueAt(int rowIndex, int columnIndex) {
      return filteredRecords.get(rowIndex);
   }

//   public boolean isColumnSortable(int col) {
//      return true;
//   }
//
//   public Comparator getComparatorForColumn(int col) {
//      switch(col) {
//         case 0:
//            return LogComparators.DATE_COMPARATOR;
//         case 1:
//            return LogComparators.LEVEL_COMPARATOR;
//         case 2:
//            return LogComparators.MESSAGE_COMPARATOR;
//      }
//      return null;
//   }
//
//   protected List getData() {
//      return filteredRecords;
//   }
}
